Type TShp1
	Global UseNormals:Int = 0
End Type

Type TShp1Header
	Global SizeOfSection:Int
	Global BatchCount:Short
	Global Pad:Short
	Global OffsetToBatches:Int
	
	Global OffsetToUnknown:Int
	Global Zero:Int
	Global OffsetToBatchAttribs:Int
	
	Global OffsetToMatrixTable:Int
	
	Global OffsetData:Int
	Global OffsetToMatrixData:Int
	Global OffsetToPacketLocations:Int
	
End Type

Type TBatch
	
	'Addition! (Keep track on array)
	Global List:TList = CreateList()
	Field Nr:Int
	'Addition! (Keep track on array)
	
	Field MatrixType:Byte
	Field Unknown2:Byte
	Field PacketCount:Short
	Field OffsetToAttribs:Short
	Field FirstMatrixData:Short
	Field FirstPacketLocation:Short
	Field Unknown3:Short
	Field Unknown4:Float
	Field bbMin:Float[3]
	Field bbMax:Float[3]
	
End Type

Type TBatchAttribs
	Field Attrib:Int
	Field DataType:Int
End Type

Type TPacketLocation
	Global Size:Int
	Global Offset:Int
End Type

Type TPrimitive
	Field PrimitiveType:Byte
	Field NumVertices:Short
End Type

Type TMatrixData
	Field Unknown1:Short
	Field Count:Short
	Field FirstIndex:Int
End Type

Function DumpPacketPrimitives( File:TStream, DataSize:Int, Attribs:Object[] )
	
	Local Done:Byte = False
	Local ReadBytes:Int = 0
	
	While Not Done
		
		Local _Type:Byte = ReadByte( File )
		ReadBytes:+1
		
		If (_Type = 0 Or Readbytes >= DataSize )
			Done = True
			Continue
		EndIf
		
		Local Count:Short = ReadShort( File )
		ReadBytes:+2
		
		Local Surf:TSurface = TSurface.Create( TBMDLoader.Mesh, GL_TRIANGLE_STRIP )
		
		For Local X:Int = 0 To Count - 1
			
			Local Vertex:TVertex = New TVertex
			Surf.DrawList.Addlast( Vertex )
			
			For Local A:TBatchAttribs = EachIn Attribs
				
				Local Val:Short
				
				Select A.DataType
					Case 1
						'u8
						Val = ReadByte( File )
						ReadBytes:+1
						
					Case 3
						'u16
						Val = ReadShort( File )
						ReadBytes:+2
						
				End Select
				
				Select A.Attrib
					Case 9
						If Val < TVtx1.Vertices.Dimensions()[0]
							'Print "Position Data: "+Val
							Vertex.PositionIndex = Val
						EndIf
					
					Case 10
						
						If Val < TVtx1.Normals.Dimensions()[0]
							TShp1.UseNormals = 1
							'Print "Normals Data: "+Val
							Vertex.NormalIndex = Val
						EndIf
					
					Default
						'Print "Attrib: "+A.Attrib
					
				End Select
				
			Next
			
		Next
		
	Wend
	
End Function

Function GetBatchAttribs:Object[]( File:TStream, Off:Int )
	
	Local OldPos:Int = StreamPos( File )
	
	Local Ret:Object[]
	Local List:TList = CreateList()
	Local A:TBatchAttribs
	
	SeekStream( File, Off )
		A:TBatchAttribs = New TBatchAttribs
		A.Attrib = ReadInt( File )
		A.DataType = ReadInt( File )
		List.Addlast( A )
		
		While A.Attrib <> 255
				
			A:TBatchAttribs = New TBatchAttribs
			A.Attrib = ReadInt( File )
			A.DataType = ReadInt( File )
			
			If A.Attrib <> 255
				List.Addlast( A )
			EndIf
				
		Wend
	SeekStream( File, OldPos )
	
	Ret = ListToArray( List )
	
	Return Ret
	
End Function

Function DumpBatch( Batch:TBatch, File:TStream, BaseOffset:Int )
	
	Local Attribs:Object[] = GetBatchAttribs( File, BaseOffset + TShp1Header.OffsetToBatchAttribs )
	
	Print "-BatchAttribs Start-"
	For Local A:TBatchAttribs = EachIn Attribs
		Print "    Attrib: "+A.Attrib
		Print "    DataType: "+A.DataType
	Next
	Print "-BatchAttribs End-"
	Print ""
	
	For A = EachIn Attribs
		If A.DataType <> 1 And A.DataType <> 3
			Print "shp1, dumpBatch(): unknown attrib data Type "+A.DataType+", skipping batch"
			Return
		EndIf
		
		Select A.Attrib
			Case 0
				'MatrixIndices
				Exit
				
			Case 9
				'Positions
				Exit
			
			Case 10
				'Normals
				Exit
				
			Case 11
			Case 12
				'colors
				Exit
			
			Case 13
			Case 14
			Case 15
			Case 16
			Case 17
			Case 18
			Case 19
			Case 20
				'tex coords
			
			Default
				Print "shp1, dumpBatch(): unknown attrib "+A.Attrib+" in batch, it might Not display correctly"
			
		End Select
		
	Next
	
	
	For Local X:Int = 0 To Batch.PacketCount - 1
		
		'Read packet location for current packet
		SeekStream( File, BaseOffset + TShp1Header.OffsetToPacketLocations + (Batch.FirstPacketLocation + X)*8 )
		TPacketLocation.Size = ReadInt( File )
		TPacketLocation.Offset = ReadInt( File )
		
		Print "-TPacket Start-"
		Print "    Size: "+TPacketLocation.Size
		Print "    Offset: "+TPacketLocation.Offset
		Print "-TPacket End-"
		Print ""
		
		
		'Read packet's primitives
		SeekStream( File, BaseOffset + TShp1Header.OffsetData + TPacketLocation.Offset )
		DumpPacketPrimitives( File, TPacketLocation.Size, Attribs )
		
	Next
	
End Function

Function DumpShp1( File:TStream, Size ) 
	
	Shp1Offset = StreamPos( File ) - 8
	
	ReadShp1Header( File, Size )
	
	SeekStream( File, Shp1Offset + TShp1Header.OffsetToBatches )
	
	For Local X:Int = 1 To 1'TShp1Header.BatchCount
		Local TempBatch:TBatch = ReadBatch( File, X-1 )
		
		Local OldPos:Int = StreamPos( File )
		DumpBatch( TempBatch, File, Shp1Offset )
		SeekStream( File, OldPos )
	Next
	
End Function

Function ReadShp1Header( File:TStream, Size )
	TShp1Header.SizeOfSection 			 = Size
	TShp1Header.BatchCount				 = ReadShort( File )
	TShp1Header.Pad						 = ReadShort( File )
	TShp1Header.OffsetToBatches			 = ReadInt( File )
	
	TShp1Header.OffsetToUnknown			 = ReadInt( File )
	TShp1Header.Zero					 = ReadInt( File )
	
	TShp1Header.OffsetToBatchAttribs	 = ReadInt( File )
	TShp1Header.OffsetToMatrixTable		 = ReadInt( File )
	TShp1Header.OffsetData				 = ReadInt( File )
	TShp1Header.OffsetToMatrixData		 = ReadInt( File )
	TShp1Header.OffsetToPacketLocations	 = ReadInt( File )
	
	Print "-Shp1Header Start-"
	Print "    SizeOfSection: " + TShp1Header.SizeOfSection
	Print "    BatchCount: " + TShp1Header.BatchCount
	Print "    Pad: " + TShp1Header.Pad
	Print "    OffsetToBatches: " + TShp1Header.OffsetToBatches
	Print ""
	Print "    OffsetToUnknown: " + TShp1Header.OffsetToUnknown
	Print "    Zero: " + TShp1Header.Zero
	Print ""
	Print "    OffsetToBatchAttribs: " + TShp1Header.OffsetToBatchAttribs
	Print "    OffsetToMatrixTable: " + TShp1Header.OffsetToMatrixTable
	Print "    OffsetData: " + TShp1Header.OffsetData
	Print "    OffsetToMatrixData: " + TShp1Header.OffsetToMatrixData
	Print "    OffsetToPacketLocations: " + TShp1Header.OffsetToPacketLocations
	Print "-Shp1Header End-"
	Print ""
	
End Function

Function ReadBatch:TBatch( File:TStream, Number:Int )
	Local A:TBatch = New TBatch
		A.Nr = Number
		A.MatrixType 		= ReadByte( File )
		A.Unknown2 			= ReadByte( File )
		A.PacketCount 		= ReadShort( File )
		A.OffsetToAttribs 	= ReadShort( File )
		A.FirstMatrixData	= ReadShort( File )
		A.FirstPacketLocation = ReadShort( File )
		A.Unknown3 			= ReadShort( File )
		A.Unknown4 			= ReadFloat( File )
		For Local X:Int = 0 To 2
			A.bbMin[X] = ReadFloat( File )
		Next
		For X = 0 To 2
			A.bbMax[X] = ReadFloat( File )
		Next
		
		Print "-New Batch-"
		Print "    MatrixType: " + A.MatrixType
		Print "    Unknown2: " + A.Unknown2
		Print "    PacketCount: " + A.PacketCount
		Print "    OffsetToAttribs: " + A.OffsetToAttribs
		Print "    FirstMatrixData: " + A.FirstMatrixData
		Print "    FirstPacketLocation: " + FirstPacketLocation
		Print "    Unknown3: " + A.Unknown3
		Print "    Unknown4: " + A.Unknown4
		Print "-End Of Batch-"
		Print ""
		
		Return A
		
End Function