Type TVtx1
	Global ArrayFormats:TArrayFormat[13]
	Global Vertices:TVector[]
	Global Normals:TVector[]
End Type

Type TVtx1Header
	Global SizeOfSection:Int
	Global ArrayFormatOffset:Int
	Global Offsets:Int[13]
End Type

Type TArrayFormat
	Field Offset:Int
	
	Field ArrayType:Int
	Field ComponentCount:Int
	Field DataType:Int
	
	Field DecimalPoint:Byte
	Field Unknown3:Byte
	Field Unknown4:Short
End Type

Function GetLength( K:Int )
	Local StartOffset = TVtx1Header.Offsets[ K ]
	For Local I:Int = K+1  To 12
		If TVtx1Header.Offsets[ I ] <> 0
			Return TVtx1Header.Offsets[ I ] - StartOffset
		EndIf
	Next
	Return TVtx1Header.SizeOfSection - StartOffset
End Function

Function ReadVertexArray( Af:TArrayFormat, File:TStream, Offset:Int, Length:Int )
	
	Local X:Int
	Local Data:Float[]
	
	SeekStream( File, Offset + AF.Offset )
	
	Select AF.DataType
		Case 3 'S16
			Print "S16 ("+(Length/2)+")"
			Data = New Float[Length/2]
			For X = 0 To (Length/2) - 1
				Data[X] = ReadSShort( File, AF.DecimalPoint )
			Next
			
		Case 4 'F32
			Print "F32 ("+(Length/4)+")"
			Data = New Float[Length/4]
			For X = 0 To (Length/4) - 1
				Data[X] = ReadFloat( File )
			Next
			
		Case 5 'RGB(A)
			Print "RGB(A)  ("+(Length)+")"
			Data = New Float[Length]
			For X = 0 To Length - 1
				Data[X] = ReadByte( File )
			Next
			
		Default
			Print "[ERROR!] VTX1 UNKNOWN DATATYPE: '"+AF.DataType+"'."
			
	End Select
	
	Select AF.ArrayType
		
		Case 9 'Positions
			Select AF.ComponentCount
				Case 0 'XY
					Print "POS XY"
					TVtx1.Vertices = New TVector[Data.Dimensions()[0]/2]
					For X = 0 To (Data.Dimensions()[0]/2) - 1
						TVtx1.Vertices[X] = TVector.Create( Data[X*2], Data[(X*2)+1], 0 )
					Next
					
				Case 1 'XYZ
					Print "POS XYZ"
					TVtx1.Vertices = New TVector[Data.Dimensions()[0]/3]
					For X = 0 To (Data.Dimensions()[0]/3) - 1
						TVtx1.Vertices[X] = TVector.Create( Data[X*3], Data[(X*3)+1], Data[(X*3)+2] )
					Next
					
			End Select
			
		Case 10 'Normals
			Select AF.ComponentCount
				Case 0 'XYZ
					Print "NORM XYZ"
					TVtx1.Normals = New TVector[Data.Dimensions()[0]/3]
					For X = 0 To (Data.Dimensions()[0]/3) - 1
						TVtx1.Normals[X] = TVector.Create( Data[X*3], Data[(X*3)+1], Data[(X*3)+2] )
					Next
					
			End Select
			
		Case 11 'Color0
		Case 12 'Color1
		Case 13, 14, 15, 16, 17, 18, 19 'Tex Coords 0-6
		Case 20 'Tex Coords 7
		
	End Select
	
End Function

Function DumpVtx1( File:TStream , Size )
	
	Local Vtx1Offset = StreamPos( File ) - 8
	
	'Read Header
	TVtx1Header.SizeOfSection = Size
	TVtx1Header.ArrayFormatOffset = ReadInt( File ) 
	
	Print "-VTX1 Header Start-"
	Print "    SizeOfSection: " + TVtx1Header.SizeOfSection
	Print "    ArrayFormatOffset: " + TVtx1Header.ArrayFormatOffset
	
	Local NumArray:Int = 0
	
	For Local X:Int = 0 To 12
		Local TempInt = ReadInt( File )
		If TempInt <> 0
			Print "    Offsets["+X+"]: "+TempInt
			TVtx1Header.Offsets[X] = TempInt
			NumArray:+1
		EndIf
	Next
	
	Print "-VTX1 Header End-"
	Print ""
	
	SeekStream( File, Vtx1Offset + TVtx1Header.ArrayFormatOffset )
	
	For X:Int = 0 To 12
		If TVtx1Header.Offsets[X] <> 0
			Local A:TArrayFormat = New TArrayFormat
				A.Offset			= TVtx1Header.Offsets[X]
				
				A.ArrayType 		= ReadInt( File ) 
				A.ComponentCount	= ReadInt( File ) 
				A.DataType 			 = ReadInt( File )
				
				A.DecimalPoint		 = ReadByte( File ) 
				A.Unknown3			 = ReadByte( File )
				A.Unknown4 			 = ReadShort( File )
				
				TVtx1.ArrayFormats[X] = A
		EndIf
	Next
	
	For X:Int = 0 To 12
		If TVtx1.ArrayFormats[X] <> Null
			Local Length:Int = GetLength( X )
			Print "Length: "+Length
			ReadVertexArray( TVtx1.ArrayFormats[X], File, Vtx1Offset, Length )
		EndIf
	Next
	
End Function

Function SignedShortToInt:Float( S:Short )
	Return (S Shl 16) Sar 16
End Function

Function ReadSShort:Float( File:TStream, DecimalPoint )
	Local S:Short = ReadShort( File )
	Return Float(SignedShortToInt( S ) / (1 Shl DecimalPoint) )
End Function

Function DivideBy( Val1:Float, Val2:Float )
	Return Int(Val1 - (Val1 Mod Val2))
End Function