
Const IMGFORMAT_I4:Int 			= 0
Const IMGFORMAT_I8:Int 			= 1
Const IMGFORMAT_A4_I4:Int 		= 2
Const IMGFORMAT_A8_I8:Int 		= 3
Const IMGFORMAT_R5_G6_B5:Int 		= 4
Const IMGFORMAT_A3_RGB5:Int		= 5
Const IMGFORMAT_ARGB8:Int 		= 6
Const IMGFORMAT_INDEX4:Int 		= 8
Const IMGFORMAT_INDEX8:Int 		= 9
Const IMGFORMAT_INDEX14_X2:Int 	= 10
Const IMGFORMAT_S3TC1:Int 		= 14

Const PAL_A8_I8:Int 	= 0
Const PAL_R5_G6_B5:Int 	= 1
Const PAL_A3_RGB5:Int 	= 2

Type TTex1
	Global Textures:TTexture[]
End Type

Type TTex1Header
	
	Global SizeOfSection:Int
	Global NumImages:Short
	Global Unknown:Short
	Global TextureHeaderOffset:Int
	Global StringTableOffset:Int
	
End Type

Type TTextureHeader
	Field Format:Byte
	Field Unknown:Byte
	Field Width:Short
	Field Height:Short
	
	Field WrapS:Byte
	Field WrapT:Byte
	
	Field Unknown3:Byte
	
	Field PaletteFormat:Byte
	Field PaletteNumEntries:Short
	Field PaletteOffset:Int
	
	Field Unknown5:Int
	
	Field MinFilter:Byte
	Field MagFilter:Byte
	
	Field Unknown7:Short
	
	Field MipMapCount:Byte
	Field Unknown8:Byte
	
	Field Unknown9:Short
	
	Field DataOffset:Int
End Type

Function ReadTextureHeader:TTextureHeader( File:TStream )
	Local A:TTextureHeader = New TTextureHeader
		A.Format 	= ReadByte( File )
		A.Unknown 	= ReadByte( File )
		A.Width 	= ReadShort( File )
		A.Height 	= ReadShort( File )
		A.WrapS 	= ReadByte( File )
		A.WrapT 	= ReadByte( File )
		A.Unknown3 	= ReadByte( File )
		A.PaletteFormat 	= ReadByte( File )
		A.PaletteNumEntries = ReadShort( File )
		A.PaletteOffset		= ReadInt( File )
		A.Unknown5			= ReadInt( File )
		A.MinFilter	= ReadByte( File )
		A.MagFilter	= ReadByte( File )
		A.Unknown7 	= ReadShort( File)
		A.MipMapCount = ReadByte( File )
		A.Unknown8 	= ReadByte( File )
		A.Unknown9 	= ReadShort( File )
		A.DataOffset= ReadInt( File )
		
		Print "-TextureHeader Start-"
		Print "    Format: "+A.Format
		Print "    Unknown: "+A.Unknown
		Print "    Width: "+A.Width
		Print "    Height: "+A.Height
		Print "    WrapS: "+A.WrapS
		Print "    WrapT: "+A.WrapT
		Print "    Unknown3: "+A.Unknown3
		Print "    PaletteFormat: "+A.PaletteFormat
		Print "    PaletteNumEntries: "+A.PaletteNumEntries
		Print "    PaletteOffset: "+A.PaletteOffset
		Print "    Unknown5: "+A.Unknown5
		Print "    MinFilter: "+A.MinFilter
		Print "    MagFilter: "+A.MagFilter		
		Print "    Unknown7: "+A.Unknown7
		Print "    MipMapCount: "+A.MipMapCount
		Print "    Unknown8: "+A.Unknown8
		Print "    Unknown9: "+A.Unknown9
		Print "    DataOffset: "+A.DataOffset
		Print "-TextureHeader End-"
		Print ""
		
		Return A
		
End Function

Function DumpTex1( File:TStream, Size:Int )
	
	Local Tex1Offset:Int = StreamPos( File ) - 8
	
	ReadTex1Header( File, Size )
	
	Local StringTable:String[] = ReadStringTable( File, Tex1Offset + TTex1Header.StringTableOffset )
	
	If StringTable.Dimensions()[0] <> TTex1Header.NumImages
		Print "Tex1: Number of strings doesn't match number of images."
	EndIf
	
	SeekStream( File, Tex1Offset + TTex1Header.TextureHeaderOffset )
	
	Local TexHeaders:TTextureHeader[ TTex1Header.NumImages ]
	
	For Local X:Int = 0 To TTex1Header.NumImages - 1
		Print StringTable[X]
		TexHeaders[X] = ReadTextureHeader( File )
	Next
	
	For X:Int = 0 To TTex1Header.NumImages - 1			
		TTex1.Textures[X] = ReadImage( File, Tex1Offset + TTex1Header.TextureHeaderOffset + TexHeaders[X].DataOffset + (X*32), TexHeaders[X].Width, TexHeaders[X].Height, TexHeaders[X].Format )
	Next
	
End Function

Function ReadImage:TTexture( File:TStream, Offset:Int, W:Int, H:Int, Format:Int )
	
	Local Texture:TTexture
	
	SeekStream( File, Offset )
	
	Local ImageData:Byte[ W*H ]
	
	For Local I:Int = 0 To (W*H)-1
		ImageData[I] = ReadByte( File )
	Next
	
	Select Format
		Case IMAGEFORMAT_I8
			Texture = ConvertI8( ImageData, W, H )
			
		Case IMAGEFORMAT_S3TC1
			ConvertS3TC1( ImageData, W, H )
			
	End Select
	
	Return Texture
	
End Function

Function ConvertS3TC1( ImageData:Byte[], W:Int, H:Int )
	
	Local TempPixMap:TPixmap = CreatePixmap( W, H, PF_RGBA8888, 1 )
	
	Local NewData:Byte[W*H]
	
	Local SI:Int = 0
	For Local Y:Int = 0 To (H/4)-1 Step 2
		For Local X:Int = 0 To (W/4)-1 Step 2
			For Local DY:Int = 0 To 1
				For Local DX:Int = 0 To 1
					If(4*(X + DX) < W And 4*(Y + DY) < H)
						
						NewData[SI] = ImageData[8*((Y + DY)*W/4 + X + DX)]
						
						SI:+1
						
					EndIf
				Next
			Next
		Next
	Next
	
	For Local K = 0 To ((W*H)/2) - 1 Step 8
		
		Swap( NewData[K], NewData[K+1] )
		Swap( NewData[K+2], NewData[K+3] )
		
    	s3tc1ReverseByte( NewData[K+4] )
    	s3tc1ReverseByte( NewData[K+5] )
    	s3tc1ReverseByte( NewData[K+6] )
    	s3tc1ReverseByte( NewData[K+7] )
		
	Next
	
	
	
	SI = 0
	For Local XP:Int = 0 To W - 1
		For Local YP:Int = 0 To H - 1
			WritePixel( TempPixMap, XP, YP, NewData[SI] )
			SI:+1
		Next
	Next
	
	'SavePixmapPNG( TempPixMap, MilliSecs()+".png" )
	
End Function

Function MemWord_LE( Array:Byte[], I:Int )
	Return Array[I] + 256 * Array[I+1]
End Function

Function MemDWord_LE( Array:Byte[], I:Int )
	Return Array[I] + 256*Array[I]
End Function

Function DecompressDXT1( ImageData:Byte[], W:Int, H:Int )
	
	Local CurrIndex:Int = 0
	
	For Local Y:Int = 0 To H - 1 Step 4
		For Local X:Int = 0 To W - 1 Step 4
			
			Local Color1:Short = MemWord_LE( ImageData, CurrIndex )
			Local Color2:Short = MemWord_LE( ImageData, CurrIndex+2 )
			Local Bits:Int = MemWord_LE( ImageData, CurrIndex+4 )
			
			CurrIndex:+8
			
			'Prepare the color Table
			Local ColorTable:Byte[4,4]
			
			
		Next
	Next
	
End Function

Function Swap( A:Byte Var, B:Byte Var )
	Local T:Byte = A
	A = B
	B = T
End Function

Function S3TC1ReverseByte:Byte( B:Byte Var )
	
	Local B1:Byte = B & 3
	Local B2:Byte = B & 12
	Local B3:Byte = B & 48
	Local B4:Byte = B & 192
	B = (b1 Shl 6) | (b2 Shl 2) | (b3 Shr 2) | (b4 Shr 6);
	
End Function

Function ConvertI8:TTexture( ImageData:Byte[], W:Int, H:Int )
	
	Local TempPixMap:TPixmap = CreatePixmap( W, H, PF_I8, 1 )
	
	Local SI:Int = 0
	For Local Y:Int = 0 To H - 1 Step 4
		For Local X:Int = 0 To W - 1 Step 8
			For Local DY:Int = 0 To 3
				For Local DX:Int = 0 To 7
					If X+DX < W And Y+DY < H
						
						WritePixel( TempPixMap, X+DX, Y+DY, ImageData[ SI ] )
						
						SI:+1
						
					EndIf
				Next
			Next
		Next
	Next
	
	Return TTexture.LoadFromPixmap( TempPixMap )
	
End Function

Function GetString:String( File:TStream, Pos:Int )
	
	Local T:Int = StreamPos( File )
	SeekStream( File, Pos )
	
	Local Ret:String
	Local C:String
	Repeat
		C = Chr(ReadByte(File))
		If C <> Chr( 0 )
			Ret = Ret + C
		Else
			Exit
		EndIf
	Forever
	
	SeekStream( File, T )
	
	Return Ret
	
End Function

Function ReadStringTable:String[]( File:TStream, Pos:Int )
	
	Local OldPos:Int = StreamPos( File )
	
	SeekStream( File, Pos )
	
	Local Count:Short = ReadShort( File )
	SeekStream( File, Pos+4 )
	
	Local StringArray:String[Count]
	
	For Local X:Int = 0 To Count - 1
		Local Unknown:Short = ReadShort( File )
		Local StringOffset:Short = ReadShort( File )
		StringArray[X] = GetString( File, Pos + StringOffset )
	Next
	
	SeekStream( File, OldPos )
	
	Return StringArray
	
End Function

Function ReadTex1Header( File:TStream, Size:Int )
	TTex1Header.SizeOfSection = Size
	TTex1Header.NumImages = ReadShort( File )
	TTex1Header.Unknown = ReadShort( File )
	TTex1Header.TextureHeaderOffset = ReadInt( File )
	TTex1Header.StringTableOffset = ReadInt( File )
	
	TTex1.Textures = New TTexture[TTex1Header.NumImages]
	
	Print "-Tex1Header Start-"
		Print "    SizeOfSection: " + TTex1Header.SizeOfSection
		Print "    NumImages: " + TTex1Header.NumImages
		Print "    Unknown: " + TTex1Header.Unknown
		Print "    TextureHeaderOffset: " + TTex1Header.TextureHeaderOffset
		Print "    StringTableOffset: " + TTex1Header.StringTableOffset
	Print "-Tex1Header End-"
	Print ""
End Function